# -*- coding: utf-8 -*-
import logging
from five import grok
from datetime import datetime

from Products.CMFCore.interfaces import IActionSucceededEvent
from Products.ATContentTypes.interfaces.interfaces import IATContentType
from zope.lifecycleevent.interfaces import IObjectModifiedEvent, IObjectCopiedEvent, IObjectCreatedEvent
#from Products.Archetypes.event import ObjectInitializedEvent
from Products.CMFEditions.Permissions import AccessPreviousVersions
from Products.CMFCore.utils import _checkPermission
from Products.CMFCore.utils import getToolByName
from Products.CMFCore.interfaces import IFolderish
from zope.component import getUtility
from plone.registry.interfaces import IRegistry
from ityou.astream.interfaces import IAstreamSettings
from .dbapi import DBApi
from . import getNotifyDBApi, isProductAvailable
from .config import NO_MESSAGES_ON_WORKFLOW_CHANGE
db = DBApi()

# ------ redis ---------------------------------------
from ityou.esi.theme.dbapi import RDBApi
rdb = RDBApi()
# ------ /redis --------------------------------------

#TODO
# @grok.subscribe(IATContentType, ObjectInitializedEvent)
# ...


@grok.subscribe(IATContentType, IObjectModifiedEvent)
def ContentModifyEventHandler(context,event):
    """Content Modified
    """
    logging.info( "\nCONTENT MODIFIED EVENT TRIGGERD: %s\n" % str(event))
    
    request     = context.REQUEST
    form        = request.form
    wt          = getToolByName(context, "portal_workflow")

    user        = request.get('AUTHENTICATED_USER')

    # TODO : Admin and private state should not be logged
    chain = wt.getChainFor(context)
    if chain:
        status = wt.getStatusOf(chain[0], context)
        #logging.info( "CHAIN / STATUS:", chain[0], status)    
        if status:
            if status.get('review_state', None) == "private":
                logging.info("\tPRIVATE")    
                return None
            elif status.get('actor', None) == "admin":
                logging.info( "\tADMIN")
                #return None
    
    if not IFolderish.providedBy(context):
        message = request.get('fieldname')
        if not message:
            utils = Utils()
            message = utils.create_or_edit(context)
            
        activity = {
          "user_id":            user.getId(),
          "user_name":          user.getProperty('fullname','').lstrip(' ').decode('utf-8'),
          "user_email":         user.getProperty('email',''),
          "content_uid":        context.UID(),
          "content_title":      context.Title().decode('utf-8'),
          "content_path":       '/'.join(context.getPhysicalPath()).decode('utf-8'),
          "message":            message
        }
        activity_id = db.addActivity(activity)
        notice = request.get('cmfeditions_version_comment')
        
        if notice:
            comment = {
                "activity_id":      activity_id,
                "user_id":          user.getId(),
                "comment":          notice.decode('utf-8'),
                "revision":         context.version_id
                       }
            if getUtility(IRegistry).forInterface(IAstreamSettings).comment_moderation:
                comment["visible"] = True
            else:
                comment["visible"] = False
                
            db.addComment(comment)
        
        notify_dbapi = getNotifyDBApi()
        if notify_dbapi:
            notify_dbapi.addNotification({
                "action":           u'astream',
                "sender_id":        activity['user_id'],
                "sender_name":      activity['user_name'],
                "sender_email":     activity['user_email'],
                "content_uid":      activity["content_uid"],
                "content_path":     activity["content_path"],
                "content_title":    activity["content_title"],
                "message":          activity["message"],
                "timestamp_mod":    datetime.now(),
                 })
        if isProductAvailable("ityou.esi.theme"):
            rdb.setStatus("astream", 1,  uid = activity['user_id'], omit_uid = True)

    return None
   
@grok.subscribe(IATContentType, IActionSucceededEvent)
def ActivityStreamWorkflowLogging(context,event):
    """ Workflow state change
    """
    logging.info( "WORKFLOW EVENT TRIGGERD: %s\n" % str(event))

    request     = context.REQUEST
    form        = request.form
    wt          = getToolByName(context, "portal_workflow")

    user        = request.get('AUTHENTICATED_USER')
    
    if not IFolderish.providedBy(context):

        # TODO : Admin and private state should not be logged
        chain = wt.getChainFor(context)
        if chain:
            status = wt.getStatusOf(chain[0], context)
            #logging.info( "CHAIN / STATUS:", chain[0], status)    
            if status:
                if status.get('review_state', None) == "private":
                    logging.info("\tPRIVATE")    
                    return None
                elif status.get('actor', None) == "admin":
                    logging.info( "\tADMIN")
                    return None

        message = request.get('workflow_action')
        if message in NO_MESSAGES_ON_WORKFLOW_CHANGE:
            return
        
        if not message:
            message = "workflow"
        
        activity = {
          "user_id":            user.getId(),
          "user_name":          user.getProperty('fullname','').lstrip(' ').decode('utf-8'),
          "user_email":         user.getProperty('email',''),
          "content_uid":        context.UID(),
          "content_title":      context.Title().decode('utf-8'),
          "content_path":       '/'.join(context.getPhysicalPath()).decode('utf-8'),
          "message":            message,
        }
        activity_id = db.addActivity(activity)

        if isProductAvailable("ityou.esi.theme"):
            ###from ityou.esi.theme.dbapi import DBApi as StatusFlagsApi
            ###sfapi = StatusFlagsApi()
            ###sfapi.setStatus("astream", True, user_negative = True, uid = activity['user_id'])   
            # ---- sqlite3 -> redis
            rdb.setStatus("astream", 1,  uid = activity['user_id'], omit_uid = True)
        
        notify_dbapi = getNotifyDBApi()
        if notify_dbapi:
            notify_dbapi.addNotification({
                "action":           u'astream',
                "sender_id":        activity['user_id'],
                "sender_name":      activity['user_name'],
                "sender_email":     activity['user_email'],
                "content_uid":      activity["content_uid"],
                "content_path":     activity["content_path"],
                "content_title":    activity["content_title"],
                "message":          activity["message"],
                "timestamp_mod":    datetime.now(),
                 })

        if isProductAvailable("ityou.esi.theme"):
            ####from ityou.esi.theme.dbapi import DBApi as StatusFlagsApi
            ####sfapi = StatusFlagsApi()
            ####sfapi.setStatus("astream", True, user_negative = True, uid = activity['user_id'])
            # ---- sqlite3 -> redis
            rdb.setStatus("astream", 1,  uid = activity['user_id'], omit_uid = True)
                #"content_path":     activity["content_path"].decode("utf-8"),#warum hier decode??
    return None


class Utils():
    def create_or_edit(self,context):
        if not _checkPermission(AccessPreviousVersions, context):
            return "undefined"

        rt = getToolByName(context, "portal_repository", None)
        if rt is None or not rt.isVersionable(context):
            return "undefined"

        context_url = context.absolute_url()
        history=rt.getHistoryMetadata(context)
        if history.getLength(context) == 1:
            return "create"
        else:
            return "edit"




# -*- coding: utf-8 -*-
from . import _
from zope.component import getUtility
from plone.registry.interfaces import IRegistry

DEBUG       = False

NO_MESSAGES_ON_WORKFLOW_CHANGE = ['hide','retract']

ACTIVITY_STREAM = '@@activity-stream'
USER_ID_BLACKLIST = ['admin']
MESSAGE_TEXT_CONTAINER = {
    'title':        _(u'has edited the title of ${icon}${tag}${name}</i>.'),
    'description':  _(u'has edited the description of ${icon}${tag}${name}s</i>.'),
    'text':         _(u'has edited the content of ${icon}${tag}${name}</i>.'),
    'create':       _(u'has created ${icon}${tag}${name}</i>.'),
    'copy':         _(u'has copied ${icon}${tag}${name}</i>.'),
    'undefined':    _(u'has edited/created ${icon}${tag}${name}</i>.'),
    'edit':         _(u'has edited ${icon}${tag}${name}</i>.')
}                   
TIME_STRING = u"%d.%m.%Y - %H:%M"
MIN_ASTREAM_DELAY = 4000
ICON_TAG = "<img class='file-icon' src='%s' />"



# -*- coding: utf-8 -*-
import os
import logging
import hashlib
from time import time
from datetime import datetime, timedelta 
from Acquisition import aq_inner
from config import DEBUG, USER_ID_BLACKLIST

# --- sqlalchemy -------------------------------
from sqlalchemy import create_engine
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy import Column, Integer, String, DateTime, Unicode, UnicodeText, Boolean
from sqlalchemy import ForeignKey
from sqlalchemy import desc
from sqlalchemy.orm import relationship, backref
from sqlalchemy.orm import sessionmaker

# fuer psql
from sqlalchemy import String, Text
# --- sqlalchemy -------------------------------

from plone.registry.interfaces import IRegistry
from zope.component import getUtility
from ityou.astream.interfaces import IAstreamSettings
from ityou.esi.theme.interfaces import IESIThemeSettings

from . import isProductAvailable    

# ------ redis ---------------------------------------
from ityou.esi.theme.dbapi import RDBApi
rdb = RDBApi()
# ------ /redis --------------------------------------

# --- psql -----------------------------------------------------
from ityou.esi.theme import PSQL_URI
# --- /psql -----------------------------------------------------

Base = declarative_base()

class Activity(Base):
    """ An activity Object
    """
    __tablename__ = 'activities'
    
    id              = Column(String, primary_key=True)
    content_path    = Column(Unicode)
    content_title   = Column(Unicode)
    user_id         = Column(Unicode)
    message         = Column(UnicodeText)
    comments        = relationship("Comment",backref="activities")
    timestamp       = Column(DateTime)

class Comment(Base):
    """ A Comment to the activities
    """
    __tablename__ = 'comments'
    
    id              = Column(Integer, primary_key=True)
    activity_id     = Column(String, ForeignKey('activities.id'))
    user_id         = Column(Unicode)
    comment         = Column(UnicodeText)
    timestamp       = Column(DateTime)
    revision        = Column(Integer)
    visible         = Column(Boolean)


class DBApi(object):
    """ DB Util
    """
    def __init__(self):
        """Initialize Database
        """
        ## --- psql ----------------------------------------------------------
        engine  = create_engine(PSQL_URI,  client_encoding='utf8', echo=False)
        ## --- /psql ---------------------------------------------------------

        self.Session = sessionmaker(bind=engine)
        self.is_esi_theme_available = True #TODO #LM isProductAvailable("ityou.esi.theme")

        Base.metadata.create_all(engine)


    def getActivities(self, \
                      context=None, \
                      timestamp=None, \
                      newer=True,  \
                      order_field=None, \
                      descending=True, \
                      user_id="", \
                      content_uid=None, \
                      content_group=True, \
                      max=50, offset=0):
        """ return the latest MAX activities
        OLD: getActivityStream + getLatestActivities
        """

        #print "================================================================="
        #print "context:", context
        #print "timestamp:", timestamp
        #print "newer:", newer
        #print "order_field:", order_field
        #print "descending:", descending
        #print "user_id:", user_id
        #print "content_uid:", content_uid
        #print "content_group:", content_group
        #print "max", max
        #print "offset", offset
        #print "================================================================="

        activities = None
        if max > 50: max = 50 # never > 50 !

        if context:
            context_path ='/'.join(context.getPhysicalPath())

        activity_list = []
        try:                
            se = self.Session()
            q = se.query(Activity)
            
            if user_id:
                q = q.filter(Activity.user_id == user_id)
                
            if timestamp:
                if newer:
                    q = q.filter(Activity.timestamp > timestamp)
                else:
                    q = q.filter(Activity.timestamp < timestamp)
                    
            if context:
                q = q.filter(Activity.content_path.like( context_path + '%'  ))

            if content_uid:
                q = q.filter(Activity.id == content_uid)
                
            if order_field:
                if descending:
                    q = q.order_by(desc(getattr(Activity,order_field)))
                else:
                    q = q.order_by(getattr(Activity,order_field))
            
            #if content_group: 
            #    #ALT SQLITE## q = q.group_by(Activity.content_uid, Activity.message)
            #    q = q.group_by(Activity.id, Activity.message, Activity.id)
                        
            activities = q[offset:offset+max]

            db_utils = DBUtils()                        
            
            for activity in activities:
                comment_list = []
                comments = activity.comments

                for comment in comments:
                    comment_list.append(db_utils.convertAttributesToDict(comment))

                act =  db_utils.convertAttributesToDict(activity)
                act['comments'] = comment_list     

                activity_list.append(act)
        
        except:
            logging.error('Error while executinh getActivities')

        finally:
            se.close()
 
        return activity_list



    def countActivities(self):
        """ return the number of activities
        OLD:checkDBStatus     
        """
        amount = None

        try:
            se = self.Session()
            amount = se.query(Activity).count()

        except:
            logging.error("Counting activities not possible")

        finally:
            se.close()

        return amount 


    def addActivity(self,na):
        """ Insert an activity in the activity database
        As this method should later be used as a webservice
        the activities have to be serialized; we cannot
        use objects as parameters 
        OLD: insertActivity
        """
        #if not DEBUG and na['user_id'] not in getUtility(IRegistry).forInterface(IAstreamSettings).blacklist:
        if not DEBUG:

            a = Activity(
                id              = na['content_uid'],
                user_id         = na['user_id'],
                content_title   = na['content_title'],
                content_path    = na['content_path'],
                message         = na['message'],
                timestamp       = datetime.now()
                )

            try:
                se = self.Session()
                se.merge(a)
                se.commit()

            except:
                logging.error("Could not insert %s" % str(a))

            finally:
                se.close()
            
        return a.id


    def getActivityComments(self, activity_id, max=50, offset=0):
        """ return the latest MAX activities
        OLD: getActivityComments
        """
        
        comments = None
        
        try:
            se = self.Session()
            a = se.query(Activity).filter(Activity.id == activity_id).one()
            comments = a.comments

        except:
            logging.error("Retrieving comment wit %s not possible" % activity_id)

        finally:
            se.close()

        if comments:
            db_utils = DBUtils()                
            return [  db_utils.convertAttributesToDict(comment) for comment in comments   ]


    def getComments(self, timestamp=None, newer=True, order_field="id", descending=True, user_id="", max=50, offset=0):
        """ returns  comments
        """
        comments = None

        if max > 50: max = 50 # never > 50 !

        try:
            se = self.Session()
            q = se.query(Comment)
            if user_id:
                q = q.filter(Comment.user_id == user_id)
            if timestamp:
                if newer:
                    q = q.filter(Comment.timestamp > timestamp)
                else:
                    q = q.filter(Comment.timestamp < timestamp)
            
            if order_field and descending:
                q = q.order_by(desc(getattr(Comment,order_field)))
            comments = q.all()[offset:offset+max]

        except:
            logging.error('Error while executing getComments')

        finally:
            se.close()

        return comments


    def addComment(self,nc):
        """ Insert an comment to the activity in the database
            Comment is a dict (nc: new comment)
            nc must be a dict because this method will
            be part of a webservice
            OLD: insertActivityComment
        """
        res = {}

        timestamp = datetime.now()
        
        c = Comment(
                activity_id     = nc['activity_id'],
                user_id         = nc['user_id'],
                comment         = nc['comment'],
                timestamp       = timestamp,
                revision        = nc["revision"],
                visible         = nc["visible"]                    
                )

        try:
            se = self.Session()

            a = se.query(Activity).filter(Activity.id == c.activity_id).one()
            a.comments.append(c)

            se.commit()
            cid = c.id            
            res = { 'id':cid, 'timestamp': str(timestamp) }

        except:
            logging.error('Error while executing addComment')

        finally:
            se.close()

        return res
            
            

    def updateComment(self,id,new_comment):
        """ updates an existing comment in the database
            Comment is a dict (nc: new comment)
            nc must be a dict because this method will
            be part of a webservice
            OLD: insertActivityComment
        """
        comment = None
        res = {}
        timestamp = datetime.now()    

        try:
            se = self.Session()
            c = se.query(Comment).filter(Comment.id == id).one()
            comment = c.comment + '\n' + new_comment['comment']
            
            c.comment = comment
            c.timestamp = timestamp
            c.visible = new_comment['visible']
                    
            se.commit()            
            res = { 'id':id, 'timestamp': str(timestamp) }
        except:
            logging.error('Error while excecuting updateCommant')

        finally:
            se.close()
        
        return res
    
    def deleteComment(self, comment_id, content_uid):
        """ Deletes comment from db with comment id
        """
        try:
            se = self.Session()
            q = se.query(Comment).filter(Comment.id == comment_id, Comment.activity_id == content_uid).delete()
            se.commit()
            res = True
        except:
            logging.error('Error while excecuting deleteCommant')
            res = False
        finally:
            se.close()

        return res
    

    def activateComment(self, comment_id, content_uid):
        """ Activates comment with comment id
        """
        try:
            se = self.Session()
            c = se.query(Comment).filter(Comment.id == comment_id, Comment.activity_id == content_uid).one()
            c.visible = True
            se.commit()
            res = True
        except:
            logging.error('Error while excecuting deleteCommant')
            res = False
        finally:
            se.close()

        return res


    def deactivateComment(self, comment_id, content_uid):
        """ Activates comment with comment id
        """
        try:
            se = self.Session()
            c = se.query(Comment).filter(Comment.id == comment_id, Comment.activity_id == content_uid).one()
            c.visible = False
            se.commit()
            res = True
        except:
            logging.error('Error while excecuting deleteCommant')
            res = False
        finally:
            se.close()

        return res


    def getLatestActivityComment(self,activity_id):
        """ Returns the latest comment to the activity, if any
        """     

        if not activity_id:
            logging.warn("getLatestActivityComment: ID of the latest activity is missing.")
            return False
           
        c = None
        try:
            se = self.Session()
            a = se.query(Activity).filter(Activity.id == activity_id).one()
            c = a.comments[-1]

        except:
            logging.info("First comment to %s " % activity_id)

        finally:
            se.close()
            
        db_utils = DBUtils()

        if c:
            return db_utils.convertAttributesToDict(c)
        else:
            return False


class DBUtils():
    """ Help functions
    """        
    def convertAttributesToDict(self, object):
        """Returns all public attributes of an object as a dict of attributes"""
        adict = dict((key, value) for key, value in object.__dict__.iteritems() 
                   if not callable(value) and not key.startswith('_'))
            
        return adict




#-*- coding: utf-8 -*-

from plone.app.registry.browser import controlpanel

from ityou.astream.interfaces import IAstreamSettings, _

class AstreamSettingsEditForm(controlpanel.RegistryEditForm):

    schema = IAstreamSettings
    label = _(u"Activity Stream settings")
    description = _(u"""Settings of the activity stream variables
    """)

    def updateFields(self):
        super(AstreamSettingsEditForm, self).updateFields()


    def updateWidgets(self):
        super(AstreamSettingsEditForm, self).updateWidgets()

class AstreamSettingsControlPanel(controlpanel.ControlPanelFormWrapper):
    form = AstreamSettingsEditForm


# -*- coding: utf-8 -*-
import logging
from time import time
from datetime import datetime, timedelta, date
import json

try:
    from BeautifulSoup import BeautifulSoup as bs
except ImportError:
    from bs4 import BeautifulSoup as bs

from stripogram import html2text

from Acquisition import aq_inner

from zope.interface import implements
from zope.component import getUtility
from zope.component.hooks import getSite

from plone.memoize.instance import memoize
from plone.app.uuid.utils import uuidToObject, uuidToCatalogBrain
from plone.registry.interfaces import IRegistry

from Products.Five.browser import BrowserView
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from Products.CMFCore.utils import getToolByName

from ityou.astream.interfaces import IAstreamSettings

from .. import _

try:
    from ityou.whoisonline import RDBApi as WhoRDBApi
    who_rdb = WhoRDBApi()
except:
    who_rdb = False

from .. import isProductAvailable

from ..config import MESSAGE_TEXT_CONTAINER, TIME_STRING, MIN_ASTREAM_DELAY, ICON_TAG
from ..dbapi import DBApi


from ityou.thumbnails.thumbnail import ThumbnailManager
tm = ThumbnailManager()

db = DBApi()

try:
    from ityou.follow.dbapi import DBApi as DBApi_follow
    dbapi_follow = DBApi_follow()
except:
    is_follow_available = False

# ------ redis ---------------------------------------
from ityou.esi.theme.dbapi import RDBApi
rdb = RDBApi()
# ------ /redis --------------------------------------

class ActivitiesView(BrowserView):
    """View of Activities
    """
    
    html_template = ViewPageTemplateFile('templates/activities.pt')
    
    def __call__(self):
        context = aq_inner(self.context)
        self.au = ActivityUtils()
        context.REQUEST.set('action', 'activities')
        self.site = getSite()
        
        ru =  getUtility(IRegistry)
        # *1000: transform milliseconds to seconds
        # We put this value in the Portlet DOM so that
        # jquery can fetch it
        # if value lower than MIN_ASTREAM_DELAY,
        # we take MIN_ASTREAM_DELAY
        self.ASTREAM_DELAY = max([ru.forInterface(IAstreamSettings).astream_delay*1000, MIN_ASTREAM_DELAY])
        self.ESI_ROOT = self.site.absolute_url() 
        
        # TODO -> 1.2
        self.user_astream = False
        self.separate_streams = False
        #################self.thumbnails_installed = isProductAvailable("ityou.thumbnails")

        mt      = getToolByName(context,"portal_membership")    
        user    = mt.getAuthenticatedMember()

        # -> MR
        self.uid = user.getId()
        return self.html_template()


    def get_activities(self,max_activities=10,max_comments=200,au_id=None):
        """ Read activities out of the Database
        returns a dict with follow keys:
          user_id (int)
          user_name
          user_email
          content_uid
          content title
          content path
          message (text)
          comments (???)
          timestamp (DateTime) 
        """    
        context = aq_inner(self.context)

        ##ToDo --- #LM
        ##uid = context.REQUEST.get('uid')

        acts = db.getActivities(context=context, user_id=au_id, max=max_activities)
        ######################################au = ActivityUtils()
        p_acts = self.au._permission_activities(acts)
        activities = self.au.convertActivitiesForTemplate(context, p_acts)
                
        logging.debug("\n====> ACTIVITIES\n\t\t\t%s" % str(activities))
        return activities
    
    def FollowInstalled(self): # Legacy???? #LM
        """ Test if ityou.follow is installed and functions are available
        """
        return isProductAvailable("ityou.follow")
            

class AjaxActivitiesView(BrowserView):
    """View of an Activity Stream loaded by AJAX and JQ
    """
    
    amount = db.countActivities()

    def __call__(self):
        """
        """
        context = aq_inner(self.context)
        request = context.REQUEST
        mt = getToolByName(context, "portal_membership")
        
        action =        request.form.get('action')
        timestamp =     request.form.get('timestamp', None)        
        uid =           request.form.get('uid')
        comment_id =    request.get('comment_id', False)
        content_uid =   request.get('content_uid', False)    

        self.au = ActivityUtils()

        if action == "get_latest_activities":
            return self.get_latest_activities(uid, timestamp, content_uid)
        elif action == "get_more_activities":
            return self.get_more_activities(uid, timestamp, content_uid)
        elif action == "get_latest_comments":
            return self.get_latest_comments(timestamp)
        elif action == "delete_comment":
            return self.delete_comment(comment_id, content_uid)
        elif action == "activate_comment":
            return self.activate_comment(comment_id, content_uid)
        elif action == "deactivate_comment":
            return self.deactivate_comment(comment_id, content_uid)
        else:
            pass
        
        return None 
            

    def get_latest_activities(self, uid, timestamp, content_uid = None):
        """ Read activities out of the Database
        returns a dict with following keys:
          user_id (int)
          user_name
          user_email
          content_uid
          content title
          content path
          message (text)
          comments (???)
          timestamp (DateTime) 
        """
        t1 = time()
            
        context = aq_inner(self.context)
                    
        # ==== search new activities ===================================
        # 1. Check if there are new activities
        #    that are not yet displayed in the browser!
        #    - read Timesstamp of the latest activity
        #      from the request 
        #    - generate select: new timestamp > latest timestamp
        # 2. Send activities back as JSON Obj.
        # --------------------------------------------------------------

        if timestamp:
            acts = db.getActivities(context=context, user_id=uid, timestamp=timestamp, content_uid=content_uid, order_field='timestamp')
        else:
            acts = db.getActivities(context=context, user_id=uid, content_uid=content_uid )

        p_acts = self.au._permission_activities(acts)

        tc = time()
        activities = self.au.convertActivitiesForTemplate(context, p_acts)
        print "\t\tDAUER    get_latest_activities (ms):", (time() - t1)*1000
        return self.au.jsonResponce(context, activities)


    def get_more_activities(self, uid, timestamp=False, content_uid = None):
        """ Read activities out of the Database
        """
        context = aq_inner(self.context)

        if not timestamp or timestamp == "":
            timestamp = datetime.now()

        acts = db.getActivities(context=context, user_id=uid, timestamp=timestamp, content_uid=content_uid, order_field='timestamp', newer=False)
        p_acts = self.au._permission_activities(acts)
        activities = self.au.convertActivitiesForTemplate(context, p_acts)

        return self.au.jsonResponce(context, activities)

        
    def get_latest_comments(self, timestamp):
        """ Read comments out of the Database
        returns a dict with following keys:
          user_id (int)
          user_name
          user_email
          comment
          timestamp (DateTime)
        """
            
        context = aq_inner(self.context)
        cos = db.getComments(timestamp=timestamp, newer=True)
        mt = getToolByName(context, "portal_membership")
        uid = mt.getAuthenticatedMember().getId()
        
        #Lm Performance killer raus #######if isProductAvailable("ityou.esi.theme"):
        ########## rdb.setStatus("astream_comments", '') # Warum wir hier der Flag gesetzt (get_latest...) #LM
       
        comments = self.au.convertCommentsForTemplate(context, cos)
        return self.au.jsonResponce(context, comments)

    def delete_comment(self, comment_id, content_uid):
        """ Deletes comment with comment id if user is author
        """
        context = aq_inner(self.context)
        obj = uuidToObject(content_uid)
        mt = getToolByName(context, "portal_membership")
        
        if obj.Creator() == mt.getAuthenticatedMember().getId():
            #return db.deleteComment(comment_id, content_uid)
            return self.au.jsonResponce(context, db.deleteComment(comment_id, content_uid) )
        else:
            return False
    
    def activate_comment(self, comment_id, content_uid):
        """ Activates comment with comment id if user is author
        """
        context = aq_inner(self.context)    
        obj = uuidToObject(content_uid)
        mt = getToolByName(context, "portal_membership")
        
        rdb.setStatus("astream_comments", 1)
        if obj.Creator() == mt.getAuthenticatedMember().getId():
            #return db.activateComment(comment_id, content_uid)
            return self.au.jsonResponce(context, db.activateComment(comment_id, content_uid) )
        else:
            return False

    def deactivate_comment(self, comment_id, content_uid):
        """ DEActivates comment with comment id if user is author
        """
        context = aq_inner(self.context)    
        obj = uuidToObject(content_uid)
        mt = getToolByName(context, "portal_membership")
        
        if obj.Creator() == mt.getAuthenticatedMember().getId():
            return self.au.jsonResponce(context, db.deactivateComment(comment_id, content_uid) )
        else:
            return False

class AjaxPostCommentView(BrowserView):
    """View for the comment posts
    Returns an activity_comment dict as json
    """

    def __call__(self):
        
        context = aq_inner(self.context)
        request = context.REQUEST
        mt      = getToolByName(context,"portal_membership")
        
        user      = mt.getAuthenticatedMember()
        user_id   = user.getId()
        user_name = user.getProperty('fullname').decode('utf-8')

        au = ActivityUtils()                
        ####LM portrait  = au.getPersonalPortrait(user_id, size='pico').absolute_url()
        ## FB
        portrait = ""
        ##
        au.getPersonalPortrait(user_id, size='pico')
        content_uid = request.get('id')
        obj = uuidToObject(content_uid)

        try:
            version_id = obj.version_id
        except:
            version_id = 0

        comment_txt = request.get("comment").replace("\n","[LF]")         
        comment = {
                'activity_id':  content_uid,
                'user_id':      user_id,      
                'user_name':    user_name,
                'user_email':   user.getProperty('email').decode('utf-8'),
                'portrait':     portrait,
                'comment':      html2text(comment_txt).replace("[LF]","\n").decode('utf-8'),
                'site_url':     getSite().absolute_url(),
                'revision':     version_id
            }

        if getUtility(IRegistry).forInterface(IAstreamSettings).comment_moderation:
            comment["visible"] = False
        else:
            comment["visible"] = True

        latest_comment = db.getLatestActivityComment(comment['activity_id'])        
        timeout = datetime.now() - timedelta(seconds=30) # 30 Sekunden werden die Comments zusammen gefasst

        if who_rdb: 
            # track online user
            who_rdb.setOnlineUser(user_id)
            # set ststus flag
            rdb.setStatus("whoisonline", 1,  uid = user_id, omit_uid = True)
        
        try:            
            if (latest_comment['user_id'] == comment['user_id']) and (latest_comment['timestamp'] > timeout):                
                comment_confirm = db.updateComment(latest_comment['id'],comment)
            else:
                comment_confirm = db.addComment(comment)
        except:
            comment_confirm = db.addComment(comment)
                            
        timestamp = datetime.strptime(comment_confirm['timestamp'], '%Y-%m-%d %H:%M:%S.%f')
        
        comment['timestamp'] = comment_confirm['timestamp']
        comment['timestr']   = timestamp.strftime(TIME_STRING)
        comment['comment']   = comment_txt.replace('[LF]','\n')
        comment['id'] = comment_confirm["id"]

        # set status flag
        rdb.setStatus("astream_comments", 1)

        return au.jsonResponce(context, comment)


class AstreamPostActivity(BrowserView):
    """ Called by Javascript when sending an activity
    """

    def __call__(self):
        #if not getUtility(IRegistry).forInterface(IAstreamSettings).allow_user_activities:
        return False
        context = aq_inner(self.context)
        request = context.REQUEST
        mt = getToolByName(context, "portal_membership")
        auth_member = mt.getAuthenticatedMember()
        
        a = {
            "user_id"         : auth_member.getId().decode("utf-8"),
            "user_name"       : auth_member.getProperty("fullname").lstrip(" ").decode("utf-8"),
            "user_email"      : auth_member.getProperty("email").decode("utf-8"),
            "content_uid"     : u"",
            "content_title"   : u"",
            "content_path"    : u"",
            "message"         : request.get("a").decode("utf-8"),
            'site_url':         getSite().absolute_url()
             }
        
        try:
            db.addActivity(a)
            return True
        except:
            return False      


class ActivityUtils():
    """small utils
    """

    def __init__(self):
        """Initialising variables
        """
        self.is_extuserprofile_available = isProductAvailable('ityou.extuserprofile')
        self.is_follow_available = isProductAvailable("ityou.follow")
        self.is_thumbnail_available = isProductAvailable("ityou.thumbnails")

    def _permission_content(self,acts):
        """ Returns only content to the user that he/she
        is allowed to see
        """        
        return [ uuidToObject(a['id']) for a in acts ]
    
    def _permission_activities(self, acts):
        """ Display only activities which are linked to
        documents the the user is allowed to see
        """
        return [ a for a in acts if uuidToCatalogBrain(a['id']) or a['id'] == ""  ]
    

    def convertActivitiesForTemplate(self, context, acts):
        """Prepares activities to be displayed in the Template
        """        
        mt = getToolByName(context,'portal_membership')
        auth_member_id = mt.getAuthenticatedMember().getId()
                
        activities = []
        tx = time()
        ###LM ???? au = ActivityUtils()

        for a in acts:
            user_id  = a['user_id']                 
            content  = uuidToObject(a['id'])

            if not content and a['id'] != "":
                continue
            
            user = mt.getMemberById(user_id)
            # es kann vorkommen dass der User gelöscht wurde #LM 2014-05-23
            if not user:
                logging.warn('Activity: user %s no longer exists!' % user_id)
                continue

            elif content:
                portal_type = content.portal_type
                content_url = content.absolute_url()
                content_creator = content.Creator()
                allow_comments = getattr(content, 'allow_discussion', False) ## True #DEBUFG content.allow_discussion 
                tag = '<i class="contenttype-%s">' % portal_type.lower()

                #LM 2014-05-13
                icon = content.getIcon()
                if icon:
                    icon_tag = ICON_TAG % (icon)
                else:
                    icon_tag = ""


                if a['content_title']:
                    content_title  = a['content_title']

                    if MESSAGE_TEXT_CONTAINER.has_key(a['message']):
                        message = _(MESSAGE_TEXT_CONTAINER[a['message']], mapping = {"name": content_title, "tag": tag, "icon": icon_tag})
                    else:
                        message = _(MESSAGE_TEXT_CONTAINER.get('undefined'), mapping = {"name": content_title, "tag": tag, "icon": icon_tag})
                else:
                    content_title  = content.title_or_id()
                    message = _(MESSAGE_TEXT_CONTAINER.get('create'), mapping = {"name": content_title, "tag": tag, "icon": icon_tag})            
            else:
                content_uid = None
                content_title = None
                content_url = None
                content_creator = None
                allow_comments = None
                message = a['message']

            # Translation: use ___context____!!!: context.translate
            message = context.translate(message)
            portrait = self.getPersonalPortrait(user_id, size="pico")
            timestr = a['timestamp'].strftime(TIME_STRING)
            
            # Adding followers param
            following = self._is_following(auth_member_id, user_id)
            """
            if self.is_follow_available:
                if dbapi_follow.isFollowing(auth_member_id, user_id):
                    following = "following"
                else:
                    following = "not-following"
            else:
                following = "no-ityou-follow" 
            """
            ##### #LM siehe oben #### user = mt.getMemberById(user_id)
                      
            a_tmpl = {
              "user_id":        user_id, 
              "portrait":       portrait,              
              "user_name":      user.getProperty("fullname", user_id),
              "user_email":     user.getProperty("email", ""),  
              "content_title":  content_title,
              "content_url":    content_url,
              "content_creator": content_creator,
              "id":             a['id'],
              "message":        message,
              "timestr":        timestr,
              "timestamp":      a['timestamp'].strftime('%Y-%m-%dT%H:%M:%S.%fZ'),
              'site_url':       getSite().absolute_url(),
              'portal_type':    portal_type,
              'follow_available': self.is_follow_available,
              'following':      following,
              'allow_comments': allow_comments,
            }
            if hasattr(content, 'version_id'):
                a_tmpl['revision'] = content.version_id
            else:
                a_tmpl['revision'] = None

            a_tmpl["thumbnails_available"] = self.is_thumbnail_available
            if a_tmpl["thumbnails_available"]:
                a_tmpl["thumbnail_url"] = tm.getCachedThumbnail(content) 

            comments = []
            
            for c in a['comments']:   

                # visible, if #LM sollte nicht hier sondern in der Abfrage gemacht werden #ToDo
                # - content Creator == authenticated user
                # - comment is your own comment
                #print "ioioioioioioioioioioioioioioioioioioioioioioioioioioioioioioio"
                #print "c['user_id']", c['user_id'], mt.getAuthenticatedMember().getId()
                #if c["visible"] ==  False \
                #                    and ( content.Creator() != mt.getAuthenticatedMember().getId() ) \
                #                    and ( c['user_id'] != mt.getAuthenticatedMember().getId() ) :
                #    continue
                if not self._display_comment( c['user_id'], content.Creator(), mt.getAuthenticatedMember().getId(), c['visible']  ):
                    continue

                uid =    c['user_id']
                user =   mt.getMemberById(uid)
                # es kann vorkommen dass der User gelöscht wurde
                if not user:
                    logging.warn('Comment: user %s no longer exists!' % uid)
                    continue

                c_tmpl = {
                    'id'  :         c["id"],
                    'user_id':      uid,
                    'user_name':    user.getProperty("fullname", uid),
                    'user_email':   user.getProperty("email", ""),
                    'comment':      c['comment'],
                    'timestamp':    c['timestamp'].strftime('%Y-%m-%dT%H:%M:%S.%fZ'),
                    'timestr':      c['timestamp'].strftime(TIME_STRING),    
                    'portrait':     self.getPersonalPortrait(uid, size="icon"), 
                    'site_url':     getSite().absolute_url() ,
                    'revision':     c['revision'],
                    'visible':      c['visible']
                }                
                comments.append(c_tmpl)
            
            a_tmpl['comments'] = comments 
            activities.append(a_tmpl)                

        return activities

    def _is_following(self,auth_member_id, user_id):
        """ returns
        "folloging", "not-following", 'no-ityou-follow'
        """
        # Adding followers param
        if self.is_follow_available:
            if dbapi_follow.isFollowing(auth_member_id, user_id):
                following = "following"
            else:
                following = "not-following"
        else:
            following = "no-ityou-follow" 
        return following


    def convertCommentsForTemplate(self, context, cos):
        """Prepares comments to be rendered in a jquery-Template
        """
        mt       = getToolByName(context,'portal_membership')
        comments = []
        
        content_objects = {}
        
        for c in cos:
            if not content_objects.has_key(c.activity_id):
                content_objects[c.activity_id] = uuidToObject(c.activity_id)

            #if c.visible == False and content_objects[c.activity_id].getOwner().getId() != mt.getAuthenticatedMember().getId():
            #    continue

            coa = content_objects[c.activity_id]
            if coa:
                if not self._display_comment( c.user_id, coa.Creator(), mt.getAuthenticatedMember().getId(), c.visible ):
                    continue

            uid  =    c.user_id
            user =    mt.getMemberById(uid)
            # es kann vorkommen dass der User gelöscht wurde
            if not user:
                logging.warn('Converting Comment: user %s no longer exists!' % uid)
                continue

            c_tmpl = {
                'id':           c.id,
                'user_id':      uid,
                'user_name':    user.getProperty("fullname", uid),
                'user_email':   user.getProperty("email", ""),
                'comment':      c.comment,
                'timestamp':    c.timestamp.strftime('%Y-%m-%dT%H:%M:%S.%fZ'),   
                'timestr':      c.timestamp.strftime(TIME_STRING),
                'portrait':     self.getPersonalPortrait(uid, size="icon"),
                'activity_id':  c.activity_id,
                'site_url':     getSite().absolute_url(),
                'revision':     c.revision,
                'visible':      c.visible
            }
            
            comments.append(c_tmpl)
                                    
        return comments

    def _display_comment(self, comment_uid, creator_id, auth_uid, visible):
        """ True, if authenticated user should see the comment,
            else False
        """

        # visible, if #LM sollte nicht hier sondern in der Abfrage gemacht werden #ToDo
        # - content Creator == authenticated user
        # - comment is your own comment
        if visible ==  False \
                            and ( creator_id != auth_uid ) \
                            and ( comment_uid != auth_uid ) :
            return False
        else:
            return True

    def jsonResponce(self, context, data):
        """ Returns Json Data in Callback function
        """
        request = context.REQUEST
        callback = request.get('callback','')        
        request.response.setHeader("Content-type","application/json")
        if callback:
            cb = callback + "(%s);"
            return cb % json.dumps(data)
        else:
            return json.dumps(data)

    @memoize    
    def getSmallPortrait(self,portrait,username,size="small-personal-portrait"):
        """ rermoves width and height attributes
            and adds a class to the img tag
            if jquery is rendering the image, we only need
            the src of the image
        """
        soup = bs(portrait)
        img = soup.img
        return str(img['src'])

    @memoize
    def getPersonalPortrait(self,id=None, verifyPermission=0, size=None):
        """Adapts the original getPersonalPortrait 
        If ityou.extpersonalportrait is installed, the
        patched personal portrai will be called, else the
        default
        Return absolute_url, not Object, need to memoize 
        for better performance
        """
        plone = getSite()
        mt = getToolByName(plone,"portal_membership")

        #if isProductAvailable('ityou.extuserprofile'):
        if self.is_extuserprofile_available:
            res = mt.getPersonalPortrait(id, size=size).absolute_url()
        else:
            res = mt.getPersonalPortrait(id=id).absolute_url()
        return res
    





# -*- coding: utf-8 -*-

# ESI GUIDED TOUR --------------------------
# from ityou.esi.theme.guide import GUIDE
# -------------------------------------------

#        'content': \
#u"""Das ist ein schöner Text  mit mehreren 
#Zeilen
#und so weiter
#""",


# ------------------------------------------------------------------------------
#
# step naming convention: http://bootstraptour.com/api/
#
# step = {
#   'element'   : ...,
#   'title'     : ...,
#   'content'   : ...,
#   'placement' : ...
# }
#
# ------------------------------------------------------------------------------


# ------------------------------------------------------------------------------
# Activity Stream
#

step_1 =  { 
        'element'   : '#personal-tools',
        'title'     : u'Persönliche Werkzeuge',
        'content'   : u'Hier finden Sie Verweise zu Ihren wichtigsten Seiten (Profil etc.).',
        'placement' : 'bottom'
    }

step_2 =  { 
        'element'   : '#notify-messages',
        'title'     : u'Persönliche Mitteilungen',
        'content'   : u'Erfahren Sie in Echtzeit sobald Ihnen jemanden eine neue Mitteilungen geschrieben hat.',
        'placement' : 'bottom'
    }
    
step_3 =  { 
        'element'   : '#notify-activities',
        'title'     : u'Letzte Aktivitäten',
        'content'   : u'Sobald jemand ein Dokument erstellt oder bearbeitet hat werden Sie in Echtzeit darüber informiert.',
        'placement' : 'bottom'
    }
    
step_4 =  { 
        'element'   : '#new-content',
        'title'     : u'Neuen Inhalt bereitstellen',
        'content'   : u'Erstellen Sie mit geringstem Aufwand neuen Inhalt und teilen diesen bei Bedarf mit den anderen Benutzer.',
        'placement' : 'bottom'
    }

step_5 =  { 
        'element'   : '#astream-tabs',
        'title'     : u'Auswahl letzter Aktivitäten',
        'content'   : u'Wechseln Sie zwischen den letzten Aktivitäten aller Benutzer oder denen Ihrer Favoriten.',
        'placement' : 'bottom'
    }


ACTIVITY = [ step_1, step_2, step_3, step_4, step_5 ]



GUIDE = {
    'activities' : ACTIVITY
}



# -*- coding: utf-8 -*-
from datetime import datetime, timedelta 
import logging

from .dbapi import DBApi

class DBTest():
    """Tests for the DB
    """
    
    def __init__(self):    
        self.db = DBApi()
        self.tdelta = datetime.now() - timedelta(seconds = 3600)

        self.test_getActivities()
        self.test_countActivities()
        self.test_addActivity()
        
        self.test_getActivityComments()
        self.test_addComment() 
        self.test_updateComment()
         
    def test_getActivities(self):
        logging.info("\n\n ACTIVITIES: \n\t\t %s \n" % str(        [ str(a) for a in self.db.getActivities(timestamp=self.tdelta) ]          ))
        
    def test_countActivities(self):
        logging.info("\n\n NUMBER OF ACTIVITIES: \n\t\t %s \n" % self.db.countActivities())
    
    def test_addActivity(self):
        act = {
               'user_id':       u'lmuller2000',
               'user_name':     u'Leo Pol',
               'user_email':    u'lm@ityou.de',
               'content_uid':   u'3434-73-6323',
               'content_title': u'Das Gaga',
               'content_path':  u'/der/gaga/',
               'message':       u'tHIS IS TE Mässäch',
               }
        
        logging.info("\n\n ADD ACTIVITY \n\t\t: %s \n" %    str(self.db.addActivity(act)))
        
    def test_addComment(self):
        com = {
               'activity_md5':  u'c7b49536316dd0a0e51c894fec6a1899',
               'user_id':       u'lmuller22',
               'user_name':     u'Luc Möller',
               'user_email':    u'info@ityou.de',
               'comment':       u'Ganz schön Tricky GAGA',
               }
        logging.info("\n\n ADD KOMMENTAR \n\t\t: %s \n" %    str(self.db.addComment(com)))
        
    def test_updateComment(self):
        logging.info("\n\n UPDATE KOMMENTAR \n\t\t: %s \n" %    self.db.updateComment(u'7cae723aee079563cb44939212edefff', u'Das ist ein neuer Kommentar<br />'))

    def test_getActivityComments(self):
        logging.info(str("\n\n KOMMENTARE \n\t\t: %s\n" %    self.db.getActivityComments('c7b49536316dd0a0e51c894fec6a1899')))
    
    def test_getLatestActivityComment(self):
        logging.info("\n\n LETZTER KOMMENTAR \n\t\t: %s\n" %    str(self.db.getLatestActivityComment('c7b49536316dd0a0e51c894fec6a1899')))    

##-------------
##db_tests = DBTest()
    



from zope.component import getMultiAdapter
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from plone.app.layout.viewlets.common import ViewletBase
from Products.CMFCore.utils import getToolByName
from Acquisition import aq_inner
     
from Products.CMFCore.interfaces._content import IFolderish

class AstreamViewlet(ViewletBase):
    
    render = ViewPageTemplateFile('browser/templates/astream_viewlet.pt')

    def available(self):
        context = aq_inner(self.context)
        ###print "context.allow_discussion", getattr(context, 'allow_discussion', False)
        try:
            if getattr(context, 'allow_discussion', False) and not IFolderish.providedBy(context):
                return True
            else:
                return False
        except:
            return False



import logging
from StringIO import StringIO
from plone.portlets.utils import unregisterPortletType
from zope.component.hooks import getSite
from zope.component import getSiteManager
from zope.component import getUtilitiesFor
from zope.component import adapts
from zope.component import queryMultiAdapter
from zope.component import queryUtility
from zope.component import getUtility
from zope.component import getMultiAdapter
from plone.portlets.interfaces import IPortletType
from plone.portlets.interfaces import IPortletManager
from plone.portlets.interfaces import IPortletManagerRenderer
from plone.portlets.interfaces import ILocalPortletAssignmentManager
from plone.portlets.interfaces import IPortletAssignmentMapping
from plone.portlets.interfaces import IPortletAssignmentSettings
from plone.portlets.interfaces import IPortletAssignment
from Products.CMFCore.utils import getToolByName

def remove_layout_properties(self):
    """Removes the layout properties of the project
    """
    portal = getSite()
    activities_layout = "activities"
    
    catalog = getToolByName(portal,'portal_catalog')
    folders = catalog(portal_type='Folder')
    for f in folders:
        folder = f.getObject()
        if folder.getLayout() == activities_layout:
            print "\t - Reset layout of %s" % f.getId 
            folder.setLayout('folder_listing')
    # Site root ------
    if portal.getLayout() == activities_layout:
        print "\t - Reset layout of root %s" % f.getId 
        portal.setLayout('folder_listing')
    
def uninstall(self):
    """We need to uninstall the views???"""
    out = StringIO()
    logging.info("Uninstalling ITYOU Activity stream" )
        
    remove_layout_properties(self)
    print >> out, "Successfully uninstalled ityou.astream"

    return out.getvalue()

    


#-*- coding: utf-8 -*-

from z3c.form import interfaces

from zope import schema
from zope.interface import Interface

from zope.i18nmessageid import MessageFactory

_ = MessageFactory('ityou.astream')

class IAstreamSettings(Interface):
    """Global activity stream settings. This describes records stored in the
    configuration registry and obtainable via plone.registry.
    """

    astream_delay = schema.Int(
            title=_(u"Time period for ajax requests"),
            description=_(u"Enter the time period between to Ajax-Requests. \
                    IMPORTANT: Be aware that time period less then 10 second may slow down your server! \
                    Time periods less then 4 seconds are not considered."),
            required=True,
            default=10,
        )

    comment_moderation = schema.Bool(
            title=_(u"Comment moderation"),
            description=_(u"Check if comments should be invisible by default \
            and can be moderated by creator of the document."),
            required=False,
            default=False,
        )

class IAstream(Interface):
    """Marker Interface
    """
    #allow_user_activities = schema.Bool(title=_(u"Status of user activities"),
    #                              description=_(u'help_user_activities',
    #                                            default=u"Should user be able to post activities to Activity Stream?"),
    #                              required=False,
    #                              default=False,)
    #
    #separate_streams = schema.Bool(title=_(u"Separate Streams"),
    #                              description=_(u'help_separate_streams',
    #                                            default=u"Should user activities appear in a separate stream?"),
    #                              required=False,
    #                              default=True,)

    #blacklist = schema.List(title=_(u"User Blacklist"),
    #                              description=_(u'help_user_blacklist',
    #                                            default=u"Of which users should no activities be generated?"),
    #                              required=False,
    #                              )

    